#!/bin/bash
set -e

# Globals
MACOS_VERSION_MAJOR=$(sw_vers -productVersion | cut -d '.' -f 1)
MACOS_VERSION_MINOR=$(sw_vers -productVersion | cut -d '.' -f 2)

if [ "${MACOS_VERSION_MAJOR}${MACOS_VERSION_MINOR}" -gt 1015 ]; then
    echo "This script does _not_ support macOS versions after 10.15!" 1>&2
    exit 9
fi

# Paths
MAIL_PATH="/Library/Mail"
BUNDLES_PATH="$MAIL_PATH/Bundles"
LOADER_BUNDLE_PATH="$BUNDLES_PATH/Mailbutler.mailbundle"
LOADER_EXECUTABLE_PATH="$LOADER_BUNDLE_PATH/Contents/MacOS/Mailbutler"
MAILBUTLER_PATH="$MAIL_PATH/Mailbutler"
BUILDS_PATH="$MAILBUTLER_PATH/Builds"
MAIL_PLIST_PATH="/Library/Preferences/com.apple.mail.plist"
MAIL_CONTAINER_PATH="$HOME/Library/Containers/com.apple.mail/Data"
USER_MAIL_PLIST_PATH="$MAIL_CONTAINER_PATH$MAIL_PLIST_PATH"
USER_PLUGIN_PLIST_PATH="$MAIL_CONTAINER_PATH/Library/Preferences/com.mailbutler.mailplugin.plist"
USER_PLUGIN_APPLICATION_SUPPORT_PATH="$MAIL_CONTAINER_PATH/Library/Application Support/com.mailbutler.mailplugin"
USER_APP_APPLICATION_SUPPORT_PATH="$HOME/Library/Application Support/com.mailbutler.app"

function usage() {
  echo -e "Usage: $0 check <version> | clean <user> | uninstall <user>"
  exit 1
}

function check() {
  PAYLOAD_BUNDLE_PATH="$BUILDS_PATH/$1/MailbutlerPayload.mailbundle"
  PAYLOAD_EXECUTABLE_PATH="$PAYLOAD_BUNDLE_PATH/Contents/MacOS/MailbutlerPayload"

  if [ -d "$MAIL_PATH" ] && [ ! -r "$MAIL_PATH" ]; then
    echo "$MAIL_PATH is not readable." 1>&2
    exit 2
  fi

  if [ ! -d "$BUNDLES_PATH" ]; then
    echo "$BUNDLES_PATH does not exist." 1>&2
    exit 2
  fi

  if [ ! -d "$BUILDS_PATH" ]; then
    echo "$BUILDS_PATH does not exist." 1>&2
    exit 2
  fi

  if [ ! -w "$BUNDLES_PATH" ]; then
    echo "$BUNDLES_PATH is not writable." 1>&2
    exit 2
  fi

  if [ ! -w "$BUILDS_PATH" ]; then
    echo "$BUILDS_PATH is not writable." 1>&2
    exit 2
  fi

  if [ -d "$LOADER_BUNDLE_PATH" ]; then
    if [ ! -r "$LOADER_EXECUTABLE_PATH" ]; then
      echo "$LOADER_EXECUTABLE_PATH is not readable." 1>&2
      exit 2
    fi

    if [ ! -w "$LOADER_BUNDLE_PATH" ]; then
      echo "$LOADER_BUNDLE_PATH is not writable." 1>&2
      exit 2
    fi
  fi

  if [ -d "$PAYLOAD_BUNDLE_PATH" ] && [ ! -r "$PAYLOAD_EXECUTABLE_PATH" ]; then
    echo "$PAYLOAD_EXECUTABLE_PATH is not readable." 1>&2
    exit 2
  fi

  if [ ! -f "$MAIL_PLIST_PATH" ]; then
    echo "$MAIL_PLIST_PATH does not exist." 1>&2
    exit 2
  fi

  set +e
  BUNDLES_ENABLED=$(defaults read $MAIL_PLIST_PATH EnableBundles)
  set -e
  if [ -z $BUNDLES_ENABLED ] || [ $BUNDLES_ENABLED -ne 1 ]; then
    echo "$MAIL_PLIST_PATH does not have EnableBundles set to 1." 1>&2
    exit 2
  fi

  if [ -r "$USER_MAIL_PLIST_PATH" ]; then
    set +e
    USER_BUNDLES_ENABLED=$(defaults read "$USER_MAIL_PLIST_PATH" EnableBundles)
    set -e
    if [ ! -z $USER_BUNDLES_ENABLED ] && [ $USER_BUNDLES_ENABLED -eq 0 ]; then
      echo "$USER_MAIL_PLIST_PATH has EnableBundles set to 0." 1>&2
      exit 2
    fi
  fi
}

function clean() {
  if [ $(id -u) -ne 0 ]; then
    echo -e "'$0 clean <user>' must be run as root."
    exit 1
  fi

  id -u "$1" 1>/dev/null # abort if user doesn't exist

  rm -Rf "$LOADER_BUNDLE_PATH"
  rm -Rf "$MAILBUTLER_PATH"

  mkdir -p "$MAIL_PATH"
  chmod u+rwx,go+rx "$MAIL_PATH"

  mkdir -p "$BUNDLES_PATH"
  chgrp staff "$BUNDLES_PATH"
  chmod g+rwx "$BUNDLES_PATH"

  mkdir -p "$BUILDS_PATH"
  chgrp -R staff "$MAILBUTLER_PATH"
  chmod -R g+rwx "$MAILBUTLER_PATH"

  # if user is not in the staff group, grant everyone read/write/execute permissions to the Bundles and Mailbutler folder
  set +e
  id -nG $1 | grep -qw staff
  GROUP_STATUS=$?
  set -e
  if [ $GROUP_STATUS -ne 0 ]; then
  chmod ugo+rwx "$BUNDLES_PATH"
  chmod -R ugo+rwx "$MAILBUTLER_PATH"
  fi

  defaults write "$MAIL_PLIST_PATH" EnableBundles 1

  if [ -r "$USER_MAIL_PLIST_PATH" ]; then
    # when changing defaults, the caller's user becomes the owner of the plist.
    # since the superuser must not become the owner of this plist, we execute
    # the command using sudo -u <user>.
    set +e
    sudo -u $1 defaults delete "$USER_MAIL_PLIST_PATH" EnableBundles
    set -e
  fi
}

function uninstall() {
  if [ $(id -u) -ne 0 ]; then
    echo -e "'$0 uninstall <user>' must be run as root."
    exit 1
  fi

  rm -Rf "$LOADER_BUNDLE_PATH"
  rm -Rf "$MAILBUTLER_PATH"
  rm -Rf "$USER_APP_APPLICATION_SUPPORT_PATH"

  if [ -r "$HOME/$MAIL_PATH" ]; then
    rm -Rf "$HOME/$LOADER_BUNDLE_PATH"
    rm -Rf "$HOME/$MAILBUTLER_PATH"
    rm -Rf "$USER_PLUGIN_APPLICATION_SUPPORT_PATH"
    set +e
    sudo -u $1 defaults delete "$USER_PLUGIN_PLIST_PATH"
    set -e
  else
    echo "[INFO] Skipping uninstallation inside the user's Mail directory because it is inaccessible. \
    Disable SIP or grant Mailbutler Full Disk Access to do a full uninstallation."
  fi

  MAILBUTLER_PATHS=$(mdfind kMDItemCFBundleIdentifier = "com.mailbutler.app")
  while read -r APP_PATH; do
    if [ ! -z "$APP_PATH" ] && [[ "$APP_PATH" == *.app ]]; then
      rm -Rf "$APP_PATH"
    fi
  done <<< "$MAILBUTLER_PATHS"
}

if [ $# -eq 2 ] && [ $1 = "uninstall" ]; then
  uninstall "$2"

elif [ $# -eq 2 ] && [ $1 = "clean" ]; then
  clean "$2"

elif [ $# -eq 2 ] && [ $1 = "check" ]; then
  check "$2"

else
  usage

fi
